<?php namespace EscapeWork\Manager\Banners\Repositories\Banner;

use EscapeWork\Manager\Repositories\BaseEloquentRepository;
use EscapeWork\Manager\Banners\Models\Banner;
use EscapeWork\Resize\Resize;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Config;
use Str;

class EloquentBannerRepository extends BaseEloquentRepository implements BannerRepository
{

    public function __construct(Banner $banner)
    {
        $this->model = $banner;
    }

    public function getAcceptedImages()
    {
        return $this->model->acceptedImages;
    }

    public function filterAndFetch($params)
    {
        $query = $this->model->with('group')->orderBy('order');

        if (isset($params['banner_group_id'])) {
            $query->where('banner_group_id', '=', $params['banner_group_id']);
        }

        return $query->get();
    }

    /**
     * Fazendo o upload do banner
     */
    public function uploadBanner($banner, $update = false)
    {
        if (! $banner instanceof UploadedFile) {
            Banner::$messages[] = 'Selecione um arquivo';
            return false;
        }

        # validando o diretório
        if (! $this->_makeDirectory()) {
            Banner::$messages[] = 'O diretório <strong>'.$this->getDir().'</strong> precisa de permissão de escrita para salvar os banners';

            return false;
        }

        # validando o tipo
        if (! $this->_validateBannerType($banner->getMimeType())) {
            Banner::$messages[] = 'Apenas imagens <strong>jpg e png</strong> são aceitas';

            return false;
        }

        # formatando o nome
        $dir      = $this->getDir();
        $filename = $this->_formatName($banner->getClientOriginalName(), $dir, $banner->guessExtension());

        # copiando a imagem
        try {
            $banner = $banner->move($dir . '/', $filename);
            $this->_createSizes($dir, $filename);

            # excluindo o arquivo atual
            if ($update) {
                $this->_deleteBanner();
            }

            $this->banner = $filename;

            return true;
        } catch(Exception $e) {
            return false;
        }
    }

    /**
     * Validando o tipo da imagem
     */
    public function _validateBannerType($type)
    {
        return in_array($type, $this->getAcceptedImages());
    }

    /**
     * Formatando o nome
     */
    protected function _formatName($name, $dir, $extension)
    {
        $name    = explode('.', $name);
        $total   = count($name);

        if ($total > 1) {
            unset($name[$total - 1]);
        }

        $name     = implode('', $name);
        $filename = Str::slug($name) . '.' . $extension;

        while (is_file($dir . '/' . $filename)) {
            $filename = '_' . $filename;
        }

        return $filename;
    }

    /**
     * Fazendo o resize da imagem
     *
     * @throws  EscapeWork\Resize\ResizeException
     */
    public function _createSizes($dir, $filename)
    {
        $sizes = Config::get('manager-banners::sizes');
        $sizes = isset($sizes[$this->banner_group_id]) ? $sizes[$this->banner_group_id] : $sizes['default'];

        foreach ($sizes as $prefix => $size) {
            $prefix = $prefix == 'default' ? '' : $prefix . '-';
            $banner = $dir . '/' . $prefix . $filename;

            # copiando o arquivo
            if (! is_file($banner)) {
                copy($dir . '/' . $filename, $banner);
            }

            # resize
            $bannerSize = getimagesize($banner);

            if ($size['width'] == $bannerSize[0] && $size['height'] == $bannerSize[1]) {
                continue;
            }

            $resize = new Resize($banner);
            $resize->setWidth($size['width'])->setHeight($size['height'])->crop();
        }
    }

    /**
     * Excluindo o arquivo antigo
     */
    public function _deleteBanner()
    {
        $dir  = $this->getDir();
        $file = $dir . '/' . $this->banner;

        if (is_file($file)) {
            unlink($file);
        }
    }

    /**
     * Criando o diretório caso não existe
     */
    public function _makeDirectory()
    {
        $dir = $this->getDir();

        if (! is_dir($dir)) {
            if (mkdir($dir, 0773)) {
                chmod($dir, 0773);
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     * Excluindo o banner
     */
    public function delete()
    {
        $this->_deleteBanner();

        return $this->model->delete();
    }

    /**
     * Retornando a URL da imagem
     */
    public function getUrl()
    {
        return $this->model->getUrl();
    }

    /**
     * Retornando a URL da imagem
     */
    public function getDir()
    {
        return $this->model->getDir();
    }

    public function getActives()
    {
        return $this->model
                    ->active()
                    ->notExpired()
                    ->orderBy('order')
                    ->get();
    }
}
